/**
 * <p>^Cg: vWFNg "Taglib&JSTL"</p>
 * <p>: -</p>
 * <p>쌠: Copyright (c) 2004 by Taka Tazawa. All Rights Reserved.</p>
 * @author Taka Tazawa
 * @since 2004/09/30
 * @version 1.0
 */
package util.dao; 

import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;

import util.dao.exception.DAOException;
import util.dao.exception.DAOFactoryException;
import util.dao.factory.RdbDAOFactory;
import util.to.ItemTO;

public class RdbItemDAO implements ItemDAO {

	private static final String SQLSELECT = "SELECT ITEM.ITEM_ID, ITEM.NAME, CATEGORY.CATEGORY_NAME, ITEM.PRICE, ITEM.DETAIL, ITEM.INVALIDATED_DATE, ITEM.LEAD_TIME, ITEM.STOCK, ITEM.REGISTRATION_DATE FROM ITEM, CATEGORY WHERE ITEM.CATEGORY_ID=CATEGORY.CATEGORY_ID ";
    private static final String SQLORDERBY = "ORDER BY ITEM_ID ";
    private static final String SQLINSERT = "INSERT INTO ITEM(NAME, CATEGORY_ID, PRICE, DETAIL, REGISTRATION_DATE) VALUES(?, ?, ?, ?, ?)";
    private static final String SQLDELETE = "DELETE FROM ITEM WHERE ITEM_ID=?";
    private static final String SQLUPDATE = "UPDATE ITEM SET NAME = ?, DETAIL = ?, CATEGORY_ID = ?, PRICE = ?, WHERE ITEM_ID = ?";
    
    private Connection conn = null;
    private PreparedStatement pstmt = null;

    public RdbItemDAO() {
    }

    public boolean deleteItem(String itemId) throws DAOException, DAOFactoryException {

    	boolean ret = false;
		try {
			conn = RdbDAOFactory.createConnection();
            pstmt = conn.prepareStatement(SQLDELETE);
			pstmt.setString(1, itemId);
            int row = pstmt.executeUpdate();
            if (row == 1) ret = true;
		} catch (SQLException e) {
            throw new DAOException("SQLException", e);
		} finally {
			try {
				if (pstmt != null) pstmt.close();
				if (conn != null) conn.close();
			} catch (SQLException e) {
                throw new DAOException(e);
            }
		}
        return ret;
    }

    public int insertItem(String name, String category, float price, String detail) throws DAOException, DAOFactoryException {

    	int ret;
		try {
			conn = RdbDAOFactory.createConnection();
            pstmt = conn.prepareStatement(SQLINSERT);
			pstmt.setString(1, name);
			pstmt.setString(2, category);
			pstmt.setFloat(3, price);
			pstmt.setString(4, detail);
			pstmt.setDate(5, new Date(System.currentTimeMillis()));
			ret = pstmt.executeUpdate();
		} catch (SQLException e) {
            throw new DAOException("SQLException", e);
		} finally {
			try {
				if (pstmt != null) pstmt.close();
				if (conn != null) conn.close();
			} catch (SQLException e) {
                throw new DAOException(e);
            }
		}
        return ret;

    }

    public boolean updateItem(String itemId, String name, String category, float price, String detail) throws DAOException, DAOFactoryException {

    	boolean ret = false;
		try {
			conn = RdbDAOFactory.createConnection();
            pstmt = conn.prepareStatement(SQLUPDATE);
			pstmt.setString(1, name);
			pstmt.setString(2, category);
			pstmt.setFloat(3, price);
			pstmt.setString(4, detail);
			pstmt.setString(5, itemId);
            int row = pstmt.executeUpdate();
            if (row == 1) ret = true;
		} catch (SQLException e) {
            throw new DAOException("SQLException", e);
		} finally {
			try {
				if (pstmt != null) pstmt.close();
				if (conn != null) conn.close();
			} catch (SQLException e) {
                throw new DAOException(e);
            }
		}
        return ret;

    }


    public Collection findAll() throws DAOException, DAOFactoryException {
        return getItemsAll();
    }

    public ItemTO findById(String itemId) throws DAOException, DAOFactoryException {

        ItemTO result = null;

        try {
			conn = RdbDAOFactory.createConnection();

            String sqlWhere = "AND ITEM.ITEM_ID = ? ";
            pstmt = conn.prepareStatement(SQLSELECT + sqlWhere + SQLORDERBY);
			pstmt.setString(1, itemId);
			pstmt.executeQuery();
			result = getItemTO(pstmt.getResultSet());
		} catch (SQLException e) {
            throw new DAOException("SQLException", e);
		} finally {
			try {
				if (pstmt != null) pstmt.close();
				if (conn != null) conn.close();
			} catch (SQLException e) {
                throw new DAOException(e);
            }
		}
		return result;
    }


	public Collection getItemsAll() throws DAOException, DAOFactoryException {

		Collection result = null;

        try {
			conn = RdbDAOFactory.createConnection();

            String sqlWhere = "";
            pstmt = conn.prepareStatement(SQLSELECT + sqlWhere + SQLORDERBY);
			pstmt.executeQuery();
			result = getItemTOs(pstmt.getResultSet());
		} catch (SQLException e) {
            throw new DAOException("SQLException", e);
		} finally {
			try {
				if (pstmt != null) pstmt.close();
				if (conn != null) conn.close();
			} catch (SQLException e) {
                throw new DAOException(e);
            }
		}
		return result;
	}

    public Collection getItemsByCategory(String category) throws DAOException, DAOFactoryException {

		Collection result = null;

		try {
			conn = RdbDAOFactory.createConnection();

            String sqlWhere = "AND ITEM.CATEGORY_ID = ? ";
            pstmt = conn.prepareStatement(SQLSELECT + sqlWhere + SQLORDERBY);
			pstmt.setString(1, category);
			pstmt.executeQuery();
			result = getItemTOs(pstmt.getResultSet());
		} catch (SQLException e) {
            throw new DAOException("SQLException", e);
		} finally {
			try {
				if (pstmt != null) pstmt.close();
				if (conn != null) conn.close();
			} catch (SQLException e) {
                throw new DAOException(e);
            }
		}
		return result;
    }

    public Collection getItemsByName(String name) throws DAOException, DAOFactoryException {

		Collection result = null;

		try {
			conn = RdbDAOFactory.createConnection();

            String sqlWhere = "AND ITEM.NAME LIKE ? ";
            pstmt = conn.prepareStatement(SQLSELECT + sqlWhere + SQLORDERBY);
			pstmt.setString(1, "%" + name + "%");
			pstmt.executeQuery();
			result = getItemTOs(pstmt.getResultSet());
		} catch (SQLException e) {
            throw new DAOException("SQLException", e);
		} finally {
			try {
				if (pstmt != null) pstmt.close();
				if (conn != null) conn.close();
			} catch (SQLException e) {
                throw new DAOException(e);
            }
		}
		return result;
    }

    public Collection getItemsByNameAndCategory(String name, String category) throws DAOException, DAOFactoryException {

		Collection result = null;

		try {
			conn = RdbDAOFactory.createConnection();

            String sqlWhere = "AND ITEM.NAME LIKE ? AND ITEM.CATEGORY_ID = ? ";
            pstmt = conn.prepareStatement(SQLSELECT + sqlWhere + SQLORDERBY);
			pstmt.setString(1, "%" + name + "%");
			pstmt.setString(2, category);
			pstmt.executeQuery();
			result = getItemTOs(pstmt.getResultSet());
		} catch (SQLException e) {
            throw new DAOException("SQLException", e);
		} finally {
			try {
				if (pstmt != null) pstmt.close();
				if (conn != null) conn.close();
			} catch (SQLException e) {
                throw new DAOException(e);
            }
		}
		return result;
    }

	private Collection getItemTOs(ResultSet rs) throws SQLException {

		Collection result = new ArrayList();
		while (rs.next()) {
			result.add(getItemTO(rs));
		}
		return result;
	}

	private ItemTO getItemTO(ResultSet rs) throws SQLException {

		ItemTO to = new ItemTO();
		to.setDetail(rs.getString("DETAIL"));
		to.setItemId(rs.getInt("ITEM_ID"));
		to.setName(rs.getString("NAME"));
		to.setCategory(rs.getString("CATEGORY_NAME"));
		to.setPrice(rs.getInt("PRICE"));
		to.setLeadTime(rs.getInt("LEAD_TIME"));
		to.setStock(rs.getInt("STOCK"));
		to.setInvalidatedDate(rs.getDate("INVALIDATED_DATE"));
		to.setRegistrationDate(rs.getDate("REGISTRATION_DATE"));
		return to;

	}

}
